@@ -18,7 +18,6 @@ import sys
 from spyder.config.base import CHECK_ALL, EXCLUDED_NAMES
 from spyder.config.fonts import MEDIUM, SANS_SERIF
 from spyder.config.utils import IMPORT_EXT
-from spyder.config.snippets import SNIPPETS
 from spyder.config.appearance import APPEARANCE
 from spyder.plugins.editor.utils.findtasks import TASKS_PATTERN
 from spyder.utils.introspection.module_completion import PREFERRED_MODULES
@@ -31,7 +30,8 @@ from spyder.utils.introspection.module_completion import PREFERRED_MODULES
 EXCLUDE_PATTERNS = ['*.csv, *.dat, *.log, *.tmp, *.bak, *.orig']
 
 # Extensions that should be visible in Spyder's file/project explorers
-SHOW_EXT = ['.py', '.ipynb', '.dat', '.pdf', '.png', '.svg']
+SHOW_EXT = ['.py', '.ipynb', '.dat', '.pdf', '.png', '.svg', '.md', '.yml',
+            '.yaml']
 
 # Extensions supported by Spyder (Editor or Variable explorer)
 USEFUL_EXT = IMPORT_EXT + SHOW_EXT
@@ -76,6 +76,24 @@ DEFAULTS = [
               'window/is_maximized': True,
               'window/is_fullscreen': False,
               'window/prefs_dialog_size': (1050, 530),
+              'use_custom_margin': True,
+              'custom_margin': 0,
+              'use_custom_cursor_blinking': False,
+              'show_internal_errors': True,
+              'check_updates_on_startup': True,
+              'cursor/width': 2,
+              'completion/size': (300, 180),
+              'report_error/remember_token': False,
+              'show_dpi_message': True,
+              }),
+            ('toolbar',
+             {
+              'enable': True,
+              'toolbars_visible': True,
+              'last_visible_toolbars': [],
+             }),
+            ('statusbar',
+             {
               'show_status_bar': True,
               'memory_usage/enable': True,
               'memory_usage/timeout': 2000,
@@ -83,23 +101,18 @@ DEFAULTS = [
               'cpu_usage/timeout': 2000,
               'clock/enable': False,
               'clock/timeout': 1000,
-              'use_custom_margin': True,
-              'custom_margin': 0,
-              'use_custom_cursor_blinking': False,
-              'show_internal_errors': True,
-              'check_updates_on_startup': True,
-              'toolbars_visible': True,
-              'cursor/width': 2,
-              'completion/size': (300, 180),
-              'report_error/remember_token': False,
-              'show_tour_message': True,
+              }),
+            ('pythonpath_manager',
+             {
+              'spyder_pythonpath': [],
               }),
             ('quick_layouts',
              {
               'place_holder': '',
-              'names': ['Matlab layout', 'Rstudio layout', 'Vertical split', 'Horizontal split'],
-              'order': ['Matlab layout', 'Rstudio layout', 'Vertical split', 'Horizontal split'],
-              'active': ['Matlab layout', 'Rstudio layout', 'Vertical split', 'Horizontal split'],
+              'names': [],
+              'order': [],
+              'active': [],
+              'ui_names': []
               }),
             ('internal_console',
              {
@@ -156,7 +169,7 @@ DEFAULTS = [
               'pdb_ignore_lib': False,
               'pdb_execute_events': True,
               'pdb_use_exclamation_mark': True,
-              'pdb_stop_first_line': True,
+              'pdb_stop_first_line': True
               }),
             ('variable_explorer',
              {
@@ -165,7 +178,7 @@ DEFAULTS = [
                                           # with ConfigParser's interpolation
               'excluded_names': EXCLUDED_NAMES,
               'exclude_private': True,
-              'exclude_uppercase': True,
+              'exclude_uppercase': False,
               'exclude_capitalized': False,
               'exclude_unsupported': False,
               'exclude_callables_and_modules': True,
@@ -213,9 +226,7 @@ DEFAULTS = [
               'tab_always_indent': False,
               'intelligent_backspace': True,
               'automatic_completions': True,
-              'automatic_completions_after_chars': 3,
-              'automatic_completions_after_ms': 300,
-              'completions_wait_for_ms': 200,
+              'automatic_completions_after_chars': 1,
               'completions_hint': True,
               'completions_hint_after_ms': 500,
               'underline_errors': False,
@@ -254,11 +265,16 @@ DEFAULTS = [
               'connect/ipython_console': False,
               'math': True,
               'automatic_import': True,
+              'plain_mode': False,
+              'rich_mode': True,
+              'show_source': False,
+              'locked': False,
               }),
             ('onlinehelp',
              {
               'enable': True,
               'zoom_factor': .8,
+              'handle_links': False,
               'max_history_entries': 20,
               }),
             ('outline_explorer',
@@ -278,16 +294,23 @@ DEFAULTS = [
               'show_all': True,
               'show_hscrollbar': True,
               'max_recent_projects': 10,
-              'visible_if_project_open': True
+              'visible_if_project_open': True,
+              'date_column': False,
+              'single_click_to_open': False,
+              'show_hidden': True,
+              'size_column': False,
+              'type_column': False,
+              'date_column': False
               }),
             ('explorer',
              {
               'enable': True,
-              'wrap': True,
               'name_filters': NAME_FILTERS,
               'show_hidden': False,
-              'show_all': True,
               'single_click_to_open': False,
+              'size_column': False,
+              'type_column': False,
+              'date_column': True
               }),
             ('find_in_files',
              {
@@ -300,12 +323,22 @@ DEFAULTS = [
               'search_text_samples': [TASKS_PATTERN],
               'more_options': False,
               'case_sensitive': False,
+              'exclude_case_sensitive': False,
               'max_results': 1000,
               }),
             ('breakpoints',
              {
               'enable': True,
               }),
+            ('completions',
+             {
+               'enable': True,
+               'enable_code_snippets': True,
+               'completions_wait_for_ms': 200,
+               'enabled_providers': {},
+               'provider_configuration': {},
+               'request_priorities': {}
+             }),
             ('profiler',
              {
               'enable': True,
@@ -313,35 +346,38 @@ DEFAULTS = [
             ('pylint',
              {
               'enable': True,
+              'history_filenames': [],
               'max_entries': 30,
+              'project_dir': None,
               }),
             ('workingdir',
              {
-              'working_dir_adjusttocontents': False,
               'working_dir_history': 20,
               'console/use_project_or_home_directory': False,
               'console/use_cwd': True,
               'console/use_fixed_directory': False,
+              'startup/use_project_or_home_directory': True,
               'startup/use_fixed_directory': False,
               }),
+            ('tours',
+             {
+              'enable': True,
+              'show_tour_message': True,
+             }),
             ('shortcuts',
              {
-              # ---- Global ----
-              # -- In app/spyder.py
+              # -- Application --
               '_/close pane': "Shift+Ctrl+F4",
               '_/lock unlock panes': "Shift+Ctrl+F5",
               '_/use next layout': "Shift+Alt+PgDown",
               '_/use previous layout': "Shift+Alt+PgUp",
-              '_/preferences': "Ctrl+Alt+Shift+P",
               '_/maximize pane': "Ctrl+Alt+Shift+M",
               '_/fullscreen mode': "F11",
               '_/save current layout': "Shift+Alt+S",
               '_/layout preferences': "Shift+Alt+P",
-              '_/show toolbars': "Alt+Shift+T",
               '_/spyder documentation': "F1",
               '_/restart': "Shift+Alt+R",
               '_/quit': "Ctrl+Q",
-              # -- In plugins/editor
               '_/file switcher': 'Ctrl+P',
               '_/symbol finder': 'Ctrl+Alt+P',
               '_/debug': "Ctrl+F5",
@@ -353,7 +389,7 @@ DEFAULTS = [
               '_/run': "F5",
               '_/configure': "Ctrl+F6",
               '_/re-run last script': "F6",
-              # -- In plugins/init
+              # -- Switch to plugin --
               '_/switch to help': "Ctrl+Shift+H",
               '_/switch to outline_explorer': "Ctrl+Shift+O",
               '_/switch to editor': "Ctrl+Shift+E",
@@ -364,17 +400,18 @@ DEFAULTS = [
               '_/switch to variable_explorer': "Ctrl+Shift+V",
               '_/switch to find_in_files': "Ctrl+Shift+F",
               '_/switch to explorer': "Ctrl+Shift+X",
-              '_/switch to plots': "Ctrl+Shift+G",
+              '_/switch to plots': "Ctrl+Shift+J" if MAC else "Ctrl+Shift+G",
               '_/switch to pylint': "Ctrl+Shift+C",
               '_/switch to profiler': "Ctrl+Shift+R",
-              # -- In widgets/findreplace.py
+              '_/switch to breakpoints': "Ctrl+Shift+B",
+              # -- Find/replace --
               'find_replace/find text': "Ctrl+F",
-              'find_replace/find next': "F3",
-              'find_replace/find previous': "Shift+F3",
+              'find_replace/find next': "Ctrl+G" if MAC else "F3",
+              'find_replace/find previous': (
+                  "Ctrl+Shift+G" if MAC else "Shift+F3"),
               'find_replace/replace text': "Ctrl+R",
               'find_replace/hide find and replace': "Escape",
-              # ---- Editor ----
-              # -- In widgets/sourcecode/codeeditor.py
+              # -- Editor --
               'editor/code completion': CTRL+'+Space',
               'editor/duplicate line up': (
                   "Ctrl+Alt+Up" if WIN else "Shift+Alt+Up"),
@@ -388,7 +425,7 @@ DEFAULTS = [
               'editor/move line up': "Alt+Up",
               'editor/move line down': "Alt+Down",
               'editor/go to new line': "Ctrl+Shift+Return",
-              'editor/go to definition': "Ctrl+G",
+              'editor/go to definition': "F3" if MAC else "Ctrl+G",
               'editor/toggle comment': "Ctrl+1",
               'editor/blockcomment': "Ctrl+4",
               'editor/unblockcomment': "Ctrl+5",
@@ -415,11 +452,12 @@ DEFAULTS = [
               'editor/paste': 'Ctrl+V',
               'editor/delete': 'Del',
               'editor/select all': "Ctrl+A",
-              # -- In widgets/editor.py
               'editor/inspect current object': 'Ctrl+I',
               'editor/breakpoint': 'F12',
               'editor/conditional breakpoint': 'Shift+F12',
               'editor/run selection': "F9",
+              'editor/run to line': 'Shift+F9',
+              'editor/run from line': CTRL + '+F9',
               'editor/go to line': 'Ctrl+L',
               'editor/go to previous file': CTRL + '+Shift+Tab',
               'editor/go to next file': CTRL + '+Tab',
@@ -455,29 +493,31 @@ DEFAULTS = [
               'editor/docstring': "Ctrl+Alt+D",
               'editor/autoformatting': "Ctrl+Alt+I",
               'editor/show in external file explorer': '',
-              # -- In Breakpoints
-              '_/switch to breakpoints': "Ctrl+Shift+B",
-              # ---- Consoles (in widgets/shell) ----
+              # -- Internal console --
               'console/inspect current object': "Ctrl+I",
               'console/clear shell': "Ctrl+L",
               'console/clear line': "Shift+Escape",
-              # ---- In Pylint ----
+              # -- Pylint --
               'pylint/run analysis': "F8",
-              # ---- In Profiler ----
+              # -- Profiler --
               'profiler/run profiler': "F10",
-              # ---- In widgets/ipythonconsole/shell.py ----
+              # -- IPython console --
               'ipython_console/new tab': "Ctrl+T",
               'ipython_console/reset namespace': "Ctrl+Alt+R",
               'ipython_console/restart kernel': "Ctrl+.",
-              # ---- In widgets/arraybuider.py ----
+              'ipython_console/inspect current object': "Ctrl+I",
+              'ipython_console/clear shell': "Ctrl+L",
+              'ipython_console/clear line': "Shift+Escape",
+              'ipython_console/enter array inline': "Ctrl+Alt+M",
+              'ipython_console/enter array table': "Ctrl+M",
+              # -- Array buider --
               'array_builder/enter array inline': "Ctrl+Alt+M",
               'array_builder/enter array table': "Ctrl+M",
-              # ---- In widgets/variableexplorer/arrayeditor.py ----
+              # -- Variable explorer --
               'variable_explorer/copy': 'Ctrl+C',
-              # ---- In widgets/variableexplorer/namespacebrowser.py ----
               'variable_explorer/search': 'Ctrl+F',
               'variable_explorer/refresh': 'Ctrl+R',
-              # ---- In widgets/plots/figurebrowser.py ----
+              # -- Plots --
               'plots/copy': 'Ctrl+C',
               'plots/previous figure': 'Ctrl+PgUp',
               'plots/next figure': 'Ctrl+PgDown',
@@ -487,67 +527,20 @@ DEFAULTS = [
               'plots/close all': 'Ctrl+Shift+W',
               'plots/zoom in': "Ctrl++",
               'plots/zoom out': "Ctrl+-",
-              # ---- In widgets/explorer ----
+              # -- Files --
               'explorer/copy file': 'Ctrl+C',
               'explorer/paste file': 'Ctrl+V',
-              'explorer/copy absolute path': 'Ctrl+Alt+C',
-              'explorer/copy relative path': 'Ctrl+Alt+Shift+C',
+              'explorer/copy absolute path': 'Alt+Shift+C',
+              'explorer/copy relative path': 'Alt+Shift+D',
+              # -- Projects --
+              'project_explorer/copy file': 'Ctrl+C',
+              'project_explorer/paste file': 'Ctrl+V',
+              'project_explorer/copy absolute path': 'Alt+Shift+C',
+              'project_explorer/copy relative path': 'Alt+Shift+D',
+              # -- Find --
+              'find_in_files/find in files': 'Alt+Shift+F',
               }),
             ('appearance', APPEARANCE),
-            ('lsp-server',
-             {
-              # This option is not used with the LSP server config
-              # It is used to disable hover hints in the editor
-              'enable_hover_hints': True,
-              'show_lsp_down_warning': True,
-              'code_completion': True,
-              'code_snippets': True,
-              'jedi_definition': True,
-              'jedi_definition/follow_imports': True,
-              'jedi_signature_help': True,
-              'preload_modules': PRELOAD_MDOULES,
-              'pyflakes': True,
-              'mccabe': False,
-              'formatting': 'autopep8',
-              'format_on_save': False,
-              'pycodestyle': False,
-              'pycodestyle/filename': '',
-              'pycodestyle/exclude': '',
-              'pycodestyle/select': '',
-              'pycodestyle/ignore': '',
-              'pycodestyle/max_line_length': 79,
-              'pydocstyle': False,
-              'pydocstyle/convention': 'numpy',
-              'pydocstyle/select': '',
-              'pydocstyle/ignore': '',
-              'pydocstyle/match': '(?!test_).*\\.py',
-              'pydocstyle/match_dir': '[^\\.].*',
-              'advanced/enabled': False,
-              'advanced/module': 'pyls',
-              'advanced/host': '127.0.0.1',
-              'advanced/port': 2087,
-              'advanced/external': False,
-              'advanced/stdio': False
-             }),
-            ('fallback-completions',
-             {
-              'enable': True,
-             }),
-            ('snippet-completions',
-             {
-               'enable': True,
-               **SNIPPETS
-             }),
-            ('kite',
-             {
-              'enable': True,
-              'call_to_action': True,
-              # Enable the installation dialog
-              'show_installation_dialog': True,
-              'show_onboarding': True,
-              'show_installation_error_message': True,
-              'spyder_runs': 1
-             }),
             ]
 
 
@@ -564,16 +557,14 @@ NAME_MAP = {
             'crash',
             'current_version',
             'historylog_filename',
-            'last_visible_toolbars',
-            'spyder_pythonpath',
             'window/position',
             'window/prefs_dialog_size',
             'window/size',
             'window/state',
             ]
          ),
-        ('appearance', [
-            'windows_style',
+        ('toolbar', [
+            'last_visible_toolbars',
             ]
          ),
         ('editor', [
@@ -591,6 +582,8 @@ NAME_MAP = {
         ('find_in_files', [
             'path_history'
             'search_text',
+            'exclude_index',
+            'search_in_index',
             ]
          ),
         ('main_interpreter', [
@@ -616,6 +609,7 @@ NAME_MAP = {
             'scrollbar_position',
           ]
          ),
+        ('pythonpath_manager', []),
         ('quick_layouts', []), # Empty list means use all options
         ('run', [
             'breakpoints',
@@ -647,4 +641,4 @@ NAME_MAP = {
 #    or if you want to *rename* options, then you need to do a MAJOR update in
 #    version, e.g. from 3.0.0 to 4.0.0
 # 3. You don't need to touch this value if you're just adding a new option
-CONF_VERSION = '63.0.0'
+CONF_VERSION = '74.0.0'
